@@ -5,12 +5,18 @@ import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
 
+# retrieve static file from package: https://stackoverflow.com/a/20885799/13697228
+from importlib.resources import open_text
+
+from sklearn.model_selection import train_test_split
 from sklearn.metrics import mean_absolute_error, roc_auc_score
 
 import torch
 from torch.optim.lr_scheduler import CyclicLR
 
-from CrabNet.utils.utils import (
+from .utils.get_compute_device import get_compute_device
+
+from .utils.utils import (
     Lamb,
     Lookahead,
     RobustL1,
@@ -20,8 +26,9 @@ from CrabNet.utils.utils import (
     DummyScaler,
     count_parameters,
 )
-from CrabNet.utils.optim import SWA
+from crabnet.utils.optim import SWA
 
+from crabnet.kingcrab import CrabNet
 
 # %%
 RNG_SEED = 42
@@ -30,11 +37,120 @@ np.random.seed(RNG_SEED)
 data_type_torch = torch.float32
 
 
+def groupby_formula(df, how="max"):
+    """Group identical compositions together and preserve original indices.
+
+    See https://stackoverflow.com/a/49216427/13697228
+
+    Parameters
+    ----------
+    df : DataFrame
+        At minimum should contain "formula" and "target" columns.
+    how : str, optional
+        How to perform the "groupby", either "mean" or "max", by default "max"
+
+    Returns
+    -------
+    DataFrame
+        The grouped DataFrame such that the original indices are preserved.
+    """
+    grp_df = (
+        df.reset_index()
+        .groupby(by="formula")
+        .agg({"index": lambda x: tuple(x), "target": "max"})
+        .reset_index()
+    )
+    return grp_df
+
+
+def data(
+    module,
+    fname="train.csv",
+    groupby=True,
+    dummy=False,
+    split=True,
+    val_size=0.2,
+    test_size=0.0,
+    random_state=42,
+):
+    """Grab data from within the subdirectories (modules) of mat_discover.
+
+    Parameters
+    ----------
+    module : Module
+        The module within mat_discover that contains e.g. "train.csv". For example,
+        from mat_discover.CrabNet.data.materials_data import elasticity
+    fname : str, optional
+        Filename of text file to open.
+    dummy : bool, optional
+        Whether to pare down the data to a small test set, by default False
+    groupby : bool, optional
+        Whether to use groupby_formula to filter identical compositions
+    split : bool, optional
+        Whether to split the data into train, val, and (optionally) test sets, by default True
+    val_size : float, optional
+        Validation dataset fraction, by default 0.2
+    test_size : float, optional
+        Test dataset fraction, by default 0.0
+    random_state : int, optional
+        seed to use for the train/val/test split, by default 42
+
+    Returns
+    -------
+    DataFrame
+        If split==False, then the full DataFrame is returned directly
+
+    DataFrame, DataFrame
+        If test_size == 0 and split==True, then training and validation DataFrames are returned.
+
+    DataFrame, DataFrame, DataFrame
+        If test_size > 0 and split==True, then training, validation, and test DataFrames are returned.
+    """
+
+    train_csv = open_text(module, fname)
+    df = pd.read_csv(train_csv)
+
+    if groupby:
+        df = groupby_formula(df, how="max")
+
+    if dummy:
+        ntot = min(100, len(df))
+        df = df.head(ntot)
+
+    if split:
+        if test_size > 0:
+            df, test_df = train_test_split(
+                df, test_size=test_size, random_state=random_state
+            )
+
+        train_df, val_df = train_test_split(
+            df, test_size=val_size / (1 - test_size), random_state=random_state
+        )
+
+        if test_size > 0:
+            return train_df, val_df, test_df
+        else:
+            return train_df, val_df
+    else:
+        return df
+
+
 # %%
 class Model:
     def __init__(
-        self, model, model_name="UnnamedModel", n_elements="infer", verbose=True
+        self,
+        model=None,
+        model_name="UnnamedModel",
+        n_elements="infer",
+        verbose=True,
+        force_cpu=False,
+        prefer_last=True,
     ):
+        if model is None:
+            compute_device = get_compute_device(
+                force_cpu=force_cpu, prefer_last=prefer_last
+            )
+            model = CrabNet(compute_device=compute_device).to(compute_device)
         self.model = model
         self.model_name = model_name
         self.data_loader = None
